#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
BannedFuncDetector - Tool for detecting banned functions in binary files
Author: Marc Rivero | @seifreed
"""

import sys

from bannedfuncdetector.bannedfunc import main

if __name__ == "__main__":
    sys.exit(main())
